/* Print information about weapns, spells, etc.
 * By using the same source as Moria proper, this gets kept in sync automatically.
 *
 * Benjamin Elijah Griffin	7 Feb 2000
 *
 * Depends:	treasure.o	for weapons list
 *		misc3.o		for attack_blows(), critical_blow()
 *		moria1.o	for test_hit()
 *		tables.o	for blows_table[][]
/*/

#include <stdio.h>
#include "config.h"
#include "constant.h"
#include "types.h"
/* #include "macros.h" */

#define MORIA_WEAPONS_C
#define extern
#include "externs.h"
#undef  extern

/* globals */
player_type py;
int randint_retmax = 0; /* a boolean: return maxvalue or minvalue in randint */
int weapon_th;
int weapon_td;
int weapon_ego;

/* externs */
extern treasure_type object_list[MAX_OBJECTS];

#define INFO_WEAPON	1
#define INFO_SPELLS	2
#define INFO_MAX	2

#define MI_LINELEN	80
#define MI_MAXSPELL	31	/* moria does not have a MAX_SPELL oddly */

/* macros */
#define	max(a,b)	( (a) > (b) ?  (a) : (b) )
#define min(a,b)	( (a) < (b) ?  (a) : (b) )
#define fabs(a)		( (a) <  0  ? -(a) : (a) )
#define calc_chance(low,high,weight) \
    max( 0., min( (high - low), \
		 (high - low) - (high - (650. + weight)) \
		 ) \
	) / 650.



/* Our fake random number generator to satisfy critical_blow() */
int
randint(num)
  int num;
{
  if ( randint_retmax ) {
    return(num);
  } else {
    return(1);
  }
}

int getint ( integer )
    int		*integer;
{
    int		tmp, i, c, p, s;

    s = p = 1;
    i = tmp = 0;
    while ((c = getchar()) != EOF) {
      if ((isspace(c) || (c == '/') || (c == '.')) && i) {
        *integer = (tmp * s);
	return(1);
      }
      if (isdigit(c)) {
        i   ++;
	tmp *= p;
	tmp += (c - '0');
	p   *= 10;
      }
      if (c == '\n') {
	return(0);
      }
      if ((c == '-') && !i) {
	s = -1;
      }
    } /* while */

  /* got EOF */
  return(-1);
}

int get_stat( stat )
    int		*stat;
{
    int		tmp, c;


    if( 1 != getint( &tmp ) )
	return( 0 );

    *stat = tmp;

    if( tmp == 18 )
    {
	c = getchar();
	if( c == '/' )
	{
	    if( 1 != getint( &tmp ) )
		return( 0 );
	    *stat += tmp;
	}
	else
	    ungetc(c, stdin);

    }

    return( 1 );
}

int FAKEcritical_blow(weight, plus, dam, player_level, player_class)
    int weight, player_level, player_class;
    double plus, dam;
{
    int critical;

    int		added_dam;
    double	chance_critical, chance_dam, total_chance;

    int		debug = 0;


    critical = dam;
    chance_critical = ( (int)(weight+5*plus+3*player_level) ) / 5000.;

    if( debug ) printf( "chance_critical=%g  chance_dam=(", chance_critical );

    chance_dam = calc_chance( weight, 400., weight );
    if( debug ) printf( "%g", chance_dam );
    total_chance = chance_dam;
    added_dam  = (2*dam +  5) * chance_dam;

    chance_dam = calc_chance( 400., 700., weight );
    if( debug ) printf( "+%g", chance_dam );
    total_chance += chance_dam;
    added_dam += (3*dam + 10) * chance_dam;

    chance_dam = calc_chance( 700., 900., weight );
    if( debug ) printf( "+%g", chance_dam );
    total_chance += chance_dam;
    added_dam += (4*dam + 15) * chance_dam;

    chance_dam = calc_chance( 900., 2000., weight );
    if( debug ) printf( "+%g)  ", chance_dam );
    total_chance += chance_dam;
    added_dam += (5*dam + 20) * chance_dam;

    if( debug ) printf( "added_dam = %d\n", added_dam );

    if( fabs( total_chance - 1. ) > 1e-14 )
    {
	printf( "ERROR:  total_chance=%g (%g)\n",
	       total_chance, total_chance - 1. );
	exit( 1 );
    }

    return( critical + added_dam * chance_critical );

}


int
getmode() {
  int  imode;
  char cmode[MI_LINELEN];

  imode = -1;
  fgets(cmode, 40, stdin);
  while ((imode < 0) || (imode > INFO_MAX)) {
    printf("Information mode: 1 quit; 2 weapons; 3 spells: ");
    fgets(cmode, 40, stdin);
    imode = atoi(cmode) - 1;
  }

  return(imode);
}

int
getego() {
  char ego[MI_LINELEN];

  fgets(ego, 40, stdin);
  /*
:r!perl -we 'foreach$a(@ARGV){foreach($a,lc($a),ucfirst(lc($a))){print qq^  if (strncmp(ego, "$_", 2) == 0) {\n    weapon_ego = SN_$a;\n    return 1;\n  }\n^}}' no HA DF SA SD SE SU FT FB
  */
  if (strncmp(ego, "no", 2) == 0) {
    weapon_ego = 0;
    return 1;
  }
  if (strncmp(ego, "no", 2) == 0) {
    weapon_ego = 0;
    return 1;
  }
  if (strncmp(ego, "No", 2) == 0) {
    weapon_ego = 0;
    return 1;
  }

  if (strncmp(ego, "HA", 2) == 0) {
    weapon_ego = SN_HA;
    return 1;
  }
  if (strncmp(ego, "ha", 2) == 0) {
    weapon_ego = SN_HA;
    return 1;
  }
  if (strncmp(ego, "Ha", 2) == 0) {
    weapon_ego = SN_HA;
    return 1;
  }
  if (strncmp(ego, "DF", 2) == 0) {
    weapon_ego = SN_DF;
    return 1;
  }
  if (strncmp(ego, "df", 2) == 0) {
    weapon_ego = SN_DF;
    return 1;
  }
  if (strncmp(ego, "Df", 2) == 0) {
    weapon_ego = SN_DF;
    return 1;
  }
  if (strncmp(ego, "SA", 2) == 0) {
    weapon_ego = SN_SA;
    return 1;
  }
  if (strncmp(ego, "sa", 2) == 0) {
    weapon_ego = SN_SA;
    return 1;
  }
  if (strncmp(ego, "Sa", 2) == 0) {
    weapon_ego = SN_SA;
    return 1;
  }
  if (strncmp(ego, "SD", 2) == 0) {
    weapon_ego = SN_SD;
    return 1;
  }
  if (strncmp(ego, "sd", 2) == 0) {
    weapon_ego = SN_SD;
    return 1;
  }
  if (strncmp(ego, "Sd", 2) == 0) {
    weapon_ego = SN_SD;
    return 1;
  }
  if (strncmp(ego, "SE", 2) == 0) {
    weapon_ego = SN_SE;
    return 1;
  }
  if (strncmp(ego, "se", 2) == 0) {
    weapon_ego = SN_SE;
    return 1;
  }
  if (strncmp(ego, "Se", 2) == 0) {
    weapon_ego = SN_SE;
    return 1;
  }
  if (strncmp(ego, "SU", 2) == 0) {
    weapon_ego = SN_SU;
    return 1;
  }
  if (strncmp(ego, "su", 2) == 0) {
    weapon_ego = SN_SU;
    return 1;
  }
  if (strncmp(ego, "Su", 2) == 0) {
    weapon_ego = SN_SU;
    return 1;
  }
  if (strncmp(ego, "FT", 2) == 0) {
    weapon_ego = SN_FT;
    return 1;
  }
  if (strncmp(ego, "ft", 2) == 0) {
    weapon_ego = SN_FT;
    return 1;
  }
  if (strncmp(ego, "Ft", 2) == 0) {
    weapon_ego = SN_FT;
    return 1;
  }
  if (strncmp(ego, "FB", 2) == 0) {
    weapon_ego = SN_FB;
    return 1;
  }
  if (strncmp(ego, "fb", 2) == 0) {
    weapon_ego = SN_FB;
    return 1;
  }
  if (strncmp(ego, "Fb", 2) == 0) {
    weapon_ego = SN_FB;
    return 1;
  }

  printf("\nChoices are: HA, DF, SA, SD, SE, SU, FT, FB, none\n");
  return 0;
}


int
getclass() {
  char class[MI_LINELEN];

  fgets(class, 40, stdin);
  if (strncmp(class, "war", 3) == 0) {
    py.misc.pclass = 0;
    return 1;
  }
  if (strncmp(class, "mag", 3) == 0) {
    py.misc.pclass = 1;
    return 1;
  }
  if (strncmp(class, "pri", 3) == 0) {
    py.misc.pclass = 2;
    return 1;
  }
  if (strncmp(class, "rog", 3) == 0) {
    py.misc.pclass = 3;
    return 1;
  }
  if (strncmp(class, "ran", 3) == 0) {
    py.misc.pclass = 4;
    return 1;
  }
  if (strncmp(class, "pal", 3) == 0) {
    py.misc.pclass = 5;
    return 1;
  }
  if (strncmp(class, "War", 3) == 0) {
    py.misc.pclass = 0;
    return 1;
  }
  if (strncmp(class, "Mag", 3) == 0) {
    py.misc.pclass = 1;
    return 1;
  }
  if (strncmp(class, "Pri", 3) == 0) {
    py.misc.pclass = 2;
    return 1;
  }
  if (strncmp(class, "Rog", 3) == 0) {
    py.misc.pclass = 3;
    return 1;
  }
  if (strncmp(class, "Ran", 3) == 0) {
    py.misc.pclass = 4;
    return 1;
  }
  if (strncmp(class, "Pal", 3) == 0) {
    py.misc.pclass = 5;
    return 1;
  }
  if (strncmp(class, "WAR", 3) == 0) {
    py.misc.pclass = 0;
    return 1;
  }
  if (strncmp(class, "MAG", 3) == 0) {
    py.misc.pclass = 1;
    return 1;
  }
  if (strncmp(class, "PRI", 3) == 0) {
    py.misc.pclass = 2;
    return 1;
  }
  if (strncmp(class, "ROG", 3) == 0) {
    py.misc.pclass = 3;
    return 1;
  }
  if (strncmp(class, "RAN", 3) == 0) {
    py.misc.pclass = 4;
    return 1;
  }
  if (strncmp(class, "PAL", 3) == 0) {
    py.misc.pclass = 5;
    return 1;
  }

  printf("\nChoices are: warrior, mage, priest, rogue, ranger, paladin\n");
  return 0;
}


void
getplayer() {
  int hold;

  printf( "Enter level of your character: " );
  while( 1 != getint( &hold ) ) {
      putchar( '\n');
  }
  py.misc.lev = hold;

  printf( "Enter class of your character: " );
  while( ! (getclass()) ) {
      putchar( '\n');
  }
  /* py.misc.pclass = hold; */

  printf( "Enter character ToHit: ");
  while( 1 != getint( &hold ) ) {
      putchar( '\n');
  }
  py.misc.dis_th = hold;

  printf( "Enter character ToDam: ");
  while( 1 != getint( &hold ) ) {
      putchar( '\n');
  }
  py.misc.dis_td = hold;

  printf( "Enter character strength: ");
  while( !get_stat( &hold ) ) {
      putchar( '\n');
  }
  py.stats.use_stat[A_STR] = hold;

  printf( "Enter character dexterity: ");
  while( !get_stat( &hold ) ) {
      putchar( '\n');
  }
  py.stats.use_stat[A_DEX] = hold;

  printf( "Enter character wisdom: ");
  while( !get_stat( &hold ) ) {
      putchar( '\n');
  }
  py.stats.use_stat[A_WIS] = hold;

  printf( "Enter character intelligence: ");
  while( !get_stat( &hold ) ) {
      putchar( '\n');
  }
  py.stats.use_stat[A_INT] = hold;

}


void
getweapon() {
  int hold;
  printf( "Enter weapon ego: ");
  while( !getego() ) {
      putchar( '\n');
  }

  printf( "Enter weapon ToHit: ");
  while( 1 != getint( &hold ) ) {
      putchar( '\n');
  }
  weapon_th = hold;

  printf( "Enter weapon ToDam: ");
  while( 1 != getint( &hold ) ) {
      putchar( '\n');
  }
  weapon_td = hold;

  printf("\nUsing player stats: str dex +TH +TD level class\n");
  printf(  "                    %3d %3d %3d %3d  %3d   %3d \n",
  	py.stats.use_stat[A_STR], py.stats.use_stat[A_DEX],
	py.misc.dis_td, py.misc.dis_th, 
	py.misc.lev, py.misc.pclass);
  printf(  "Using weapon stats: ego     +TH +TD\n");
  printf(  "                    %5s   %3d %3d\n",
  	special_names[weapon_ego], weapon_th, weapon_td);
}

int
main (ac, av) 
  int ac;
  char *av[];
{
  int		i, j;
  int		infomode;

  int		num_spells = -1; 	/* below the lower bound */
  int		spells[MI_MAXSPELL];
  int		spelloffset;
  int		failure;
  spell_type	*s_ptr;

  int		num_weapons = MAX_OBJECTS; /* an upper bound */
  int		weapons[MAX_OBJECTS];
  int		weapondam[MAX_OBJECTS];
  int		bestweapons[MAX_OBJECTS];
  int		egobonus, egomod;
  int		max_blows, real_blows;
  int		max_dam, avg_dam;
  int		adj_weight;
  double	max_dam_d, avg_dam_d, critical;
  char		*ego, egonone; 

  getplayer();

  while (infomode = getmode()) {
    switch (infomode) {

      case INFO_SPELLS:
	/* Find spells. Afterwards we will know the real value of num_spells */
	if (class[py.misc.pclass].spell == NONE) {
	  num_spells = 0;
	} else
	if (num_spells == -1) {
	  for (j = i = 0; i < MI_MAXSPELL; i++) {
	    /* no spells for warriors, so pclass - 1 */
	    if (magic_spell[py.misc.pclass-1][i].slevel <= MAX_PLAYER_LEVEL) {
	      spells[j] = i;
	      j++;
	    }
	  }
	  num_spells = j;
        }

	spelloffset = (class[py.misc.pclass].spell == MAGE)? 0 : MI_MAXSPELL;

	if (num_spells == 0) {
	  printf("There are no spells available to you.\n");
	} else {
	  printf("%24s %5s %5s %5s %s\n", "Spell", "Level", "Mana", "%Fail",
	  	 "1st UseExp");

	  for (i = 0; i < num_spells; i++) {
	    j = spells[i];
	    s_ptr = &magic_spell[py.misc.pclass-1][j];

	    failure = s_ptr->sfail - 3*(py.misc.lev - s_ptr->slevel);
	    if (failure <  5) { failure =  5; } 
	    if (failure > 95) { failure = 95; } 
	    printf("%24s", spell_names[spelloffset + j]);
	    printf("%5d %5d", s_ptr->slevel, s_ptr->smana);
	    printf("%5d %5d\n", failure, s_ptr->sexp * 4);

	  }
	}
        break;

	/* end INFO_SPELLS */

      case INFO_WEAPON:
	if (num_weapons == MAX_OBJECTS) {
	  /* Find weapons. Afterwards we will know the real value of num_weapons */
	  for (j = i = 0; i < num_weapons; i++) {
	    int tval = (object_list[i].tval);
	    switch(tval) {
	      case TV_SWORD:
	      case TV_HAFTED:
	      case TV_POLEARM:
	      case TV_DIGGING:	/* shovels and picks are weapons, although poor */
		weapons[j] = i;
		j++;
	    }
	  } /* for each raw object */
	  num_weapons = j;
        }

	egonone  = '\0';
	switch(weapon_ego) {
	  case SN_DF: ego = special_names[SN_DF]; egobonus = 1; egomod = 1; break;
	  case SN_FB: ego = special_names[SN_FB]; egobonus = 3; egomod = 2; break;
	  case SN_FT: ego = special_names[SN_FT]; egobonus = 3; egomod = 2; break;
	  case SN_HA: ego = special_names[SN_HA]; egobonus = 3; egomod = 1; break;
	  case SN_SA: ego = special_names[SN_SA]; egobonus = 2; egomod = 1; break;
	  case SN_SD: ego = special_names[SN_SD]; egobonus = 4; egomod = 1; break;
	  case SN_SU: ego = special_names[SN_SU]; egobonus = 3; egomod = 1; break;
	  case SN_SE: ego = special_names[SN_SE]; egobonus = 2; egomod = 1; break;
	     default: ego = &egonone;             egobonus = 0; egomod = 1;
	}

	putchar('\n');
	printf("                                               |Max|Cur|Trn|Ego|Sin|    |Base\n");
	printf("                                        Weapon |Hit|Hit|Dam|Dam|Dam| Lbs| $ \n");
	for (i = 0; i < num_weapons; i++) {
	  int			num = weapons[i];
	  int			tot_tohit = py.misc.dis_th + weapon_th;
	  int			tot_todam; /* = py.misc.dis_td + weapon_td; */
	  int			realstr   = py.stats.use_stat[A_STR];
	  int			realdex   = py.stats.use_stat[A_DEX];
	  treasure_type	*weapon_ptr = &object_list[num];
	  char		*des, item[MI_LINELEN], lbs[8];
	  int			k, j;


	  /* The difference between max_blows and real_blows is the
	  /* player's strength/dexterity.
	   */
	  real_blows = attack_blows((int)weapon_ptr->weight, &tot_tohit);
	  py.stats.use_stat[A_STR] = 118;
	  py.stats.use_stat[A_DEX] = 118;
	  max_blows  = attack_blows((int)weapon_ptr->weight, &tot_tohit);
	  py.stats.use_stat[A_STR] = realstr;
	  py.stats.use_stat[A_DEX] = realdex;

	  k          = ((int)weapon_ptr->damage[0] * (int)weapon_ptr->damage[1]);
	  avg_dam    = (real_blows * k);
	  max_dam    = avg_dam * egobonus / egomod;

	  if (weapon_ptr->name[0] == '&') {
	    /* skip leading "& " */
	    des = weapon_ptr->name + 2;
	  } else {
	    des = weapon_ptr->name;
	  }

	  if (weapon_th || weapon_td) {
	    sprintf(item, "%s(%dd%d)[%d,%d]%s", des,
		weapon_ptr->damage[0], weapon_ptr->damage[1], 
		weapon_th, weapon_td, ego);
	  } else {
	    sprintf(item, "%s(%dd%d)%s", des,
		weapon_ptr->damage[0], weapon_ptr->damage[1], ego);
	  }

	  for (j = 0; j < 8; j++) { lbs[j] = '\0'; }

	  sprintf(lbs, "%4.1f", (weapon_ptr->weight/10.));

	  /*     name   mhit chit trn ego hit lb $$$  */
	  printf("%46s | %d | %d |%3d|%3d|%3d|%s|%3d\n",
	      item,
	      max_blows, real_blows,
	      avg_dam, max_dam, k,
	      (lbs), weapon_ptr->weight);

	} /* for each known weapon */
	break;
	/* end INFO_WEAPONS */

      default:
        printf("Oops, mode %d isn't caught.\n", infomode);
        break;

    } /* switch (infomode) */
  } /* while info mode */

  return(0);
} /* main */

